package game

import (
	"net"
	"strings"
)

// Chat is used for chat messages.
func (game *Game) Chat(data []byte, address *net.UDPAddr) error {
	player := game.players.ByAddress(address)

	if player == nil {
		return ErrUnknownAddress
	}

	message := string(data)

	if game.ChatCommand(player, message) {
		return nil
	}

	newData := []byte{}
	newData = AppendString(newData, player.ID)
	newData = AppendString(newData, message)
	game.Broadcast(Chat, newData)
	return nil
}

// ChatCommand executes chat commands and returns true if it was one.
func (game *Game) ChatCommand(player *Player, message string) bool {
	if !strings.HasPrefix(message, "/") {
		return false
	}

	switch message {
	case "/logout":
		player.Send(Logout, nil)
		game.players.Remove(player)
	}

	return true
}
