package game_test

import (
	"server/game"
	"testing"
)

var (
	code = byte(1)
	data = []byte("Hello")
)

func noop([]byte) {}

func BenchmarkSendAppend(b *testing.B) {
	b.ReportAllocs()

	for i := 0; i < b.N; i++ {
		tmp := append([]byte{code}, data...)
		noop(tmp)
	}
}

func BenchmarkSendCopy(b *testing.B) {
	b.ReportAllocs()

	for i := 0; i < b.N; i++ {
		tmp := make([]byte, len(data)+1)
		tmp[0] = code
		copy(tmp[1:], data)
		noop(tmp)
	}
}

func BenchmarkReceivePointer(b *testing.B) {
	b.ReportAllocs()
	incoming := make(chan *game.Packet, 1)

	for i := 0; i < b.N; i++ {
		incoming <- &game.Packet{data, nil}
		packet := <-incoming

		if packet.Data[0] != 'H' {
			b.FailNow()
		}
	}
}

func BenchmarkReceiveValue(b *testing.B) {
	b.ReportAllocs()
	incoming := make(chan game.Packet, 1)
	data := []byte("Hello")

	for i := 0; i < b.N; i++ {
		incoming <- game.Packet{data, nil}
		packet := <-incoming

		if packet.Data[0] != 'H' {
			b.FailNow()
		}
	}
}
